/**
 * router-core
 *
 * Copyright (c) TanStack
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.md file in the root directory of this source tree.
 *
 * @license MIT
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var invariant = require('tiny-invariant');
var store = require('./store.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var invariant__default = /*#__PURE__*/_interopDefaultLegacy(invariant);

// RouterAction is a constrained identify function that takes options: key, action, onSuccess, onError, onSettled, etc
function createAction(options) {
  const store$1 = store.createStore({
    submissions: []
  }, options.debug);
  return {
    options,
    store: store$1,
    reset: () => {
      store$1.setState(s => {
        s.submissions = [];
      });
    },
    submit: async payload => {
      const submission = {
        submittedAt: Date.now(),
        status: 'pending',
        payload: payload,
        invalidate: () => {
          setSubmission(s => {
            s.isInvalid = true;
          });
        },
        getIsLatest: () => store$1.state.submissions[store$1.state.submissions.length - 1]?.submittedAt === submission.submittedAt
      };
      const setSubmission = updater => {
        store$1.setState(s => {
          const a = s.submissions.find(d => d.submittedAt === submission.submittedAt);
          invariant__default["default"](a, 'Could not find submission in store');
          updater(a);
        });
      };
      store$1.setState(s => {
        s.submissions.push(submission);
        s.submissions.reverse();
        s.submissions = s.submissions.slice(0, options.maxSubmissions ?? 10);
        s.submissions.reverse();
      });
      const after = async () => {
        options.onEachSettled?.(submission);
        if (submission.getIsLatest()) await options.onLatestSettled?.(submission);
      };
      try {
        const res = await options.action?.(submission.payload);
        setSubmission(s => {
          s.response = res;
        });
        await options.onEachSuccess?.(submission);
        if (submission.getIsLatest()) await options.onLatestSuccess?.(submission);
        await after();
        setSubmission(s => {
          s.status = 'success';
        });
        return res;
      } catch (err) {
        console.error(err);
        setSubmission(s => {
          s.error = err;
        });
        await options.onEachError?.(submission);
        if (submission.getIsLatest()) await options.onLatestError?.(submission);
        await after();
        setSubmission(s => {
          s.status = 'error';
        });
        throw err;
      }
    }
  };
}

exports.createAction = createAction;
//# sourceMappingURL=actions.js.map
