/**
 * router-core
 *
 * Copyright (c) TanStack
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.md file in the root directory of this source tree.
 *
 * @license MIT
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var immer = require('immer');

immer.setAutoFreeze(false);
let queue = [];
let batching = false;
function flush() {
  if (batching) return;
  queue.forEach(cb => cb());
  queue = [];
}
function createStore(initialState, debug) {
  const listeners = new Set();
  const store = {
    state: initialState,
    subscribe: listener => {
      listeners.add(listener);
      return () => listeners.delete(listener);
    },
    setState: updater => {
      const previous = store.state;
      store.state = immer.produce(d => {
        updater(d);
      })(previous);
      if (debug) console.log(store.state);
      queue.push(() => listeners.forEach(listener => listener(store.state, previous)));
      flush();
    }
  };
  return store;
}
function batch(cb) {
  batching = true;
  cb();
  batching = false;
  flush();
}

exports.batch = batch;
exports.createStore = createStore;
//# sourceMappingURL=store.js.map
